'use strict';

var path = require('path');
var parse = require('./parsers/index');

var sourceMapsCorrections = {};
var DEFAULT_OPTIONS = {
  moduleName: 'styled-components'
};

module.exports = function (options) {
  return {
    // Get string for stylelint to lint
    code(input, filepath) {
      try {
        var absolutePath = path.resolve(process.cwd(), filepath);
        sourceMapsCorrections[absolutePath] = {};

        var _parse = parse(input, absolutePath, Object.assign({}, DEFAULT_OPTIONS, options)),
            extractedCSS = _parse.extractedCSS,
            sourceMap = _parse.sourceMap;
        // Save source location, merging existing corrections with current corrections


        sourceMapsCorrections[absolutePath] = Object.assign(sourceMapsCorrections[absolutePath], sourceMap);
        return extractedCSS;
      } catch (e) {
        // incorrect interpolations will throw CssSyntaxError and they'll be handled by stylelint
        if (e.name === 'CssSyntaxError') {
          throw e;
        }
        return '';
      }
    },
    // Fix sourcemaps
    result(stylelintResult, filepath) {
      var lineCorrection = sourceMapsCorrections[filepath];
      var warnings = stylelintResult.warnings.map(function (warning) {
        return Object.assign({}, warning, {
          // Replace "brace" with "backtick" in warnings, e.g.
          // "Unexpected empty line before closing backtick" (instead of "brace")
          text: warning.text.replace(/brace/, 'backtick'),
          line: lineCorrection[warning.line]
        });
      });

      return Object.assign({}, stylelintResult, { warnings });
    }
  };
};